@@ -5,6 +5,8 @@ SPDX-License-Identifier: MIT-0
 
 from __future__ import annotations
 
+import hashlib
+import json
 import logging
 import random
 import string
@@ -156,7 +158,7 @@ class _Transform:
                 elif k == "Fn::FindInMap":
                     try:
                         mapping = _ForEachValueFnFindInMap(get_hash(v), v)
-                        map_value = mapping.value(cfn, params, True)
+                        map_value = mapping.value(cfn, params, True, False)
                         # if we get None this means its all strings but couldn't be resolved
                         # we will pass this forward
                         if map_value is None:
@@ -167,6 +169,9 @@ class _Transform:
                     except Exception as e:  # pylint: disable=broad-exception-caught
                         # We couldn't resolve the FindInMap so we are going to leave it as it is
                         LOGGER.debug("Transform and Fn::FindInMap error: %s", {str(e)})
+                        for i, el in enumerate(v):
+                            v[i] = self._walk(el, params, cfn)
+                        obj[k] = v
                 elif k == "Ref":
                     if isinstance(v, str):
                         if v in params:
@@ -193,7 +198,9 @@ class _Transform:
         return obj
 
     def _replace_string_params(
-        self, s: str, params: Mapping[str, Any]
+        self,
+        s: str,
+        params: Mapping[str, Any],
     ) -> Tuple[bool, str]:
         pattern = r"(\$|&){[a-zA-Z0-9\.:]+}"
         if not re.search(pattern, s):
@@ -201,6 +208,8 @@ class _Transform:
 
         new_s = deepcopy(s)
         for k, v in params.items():
+            if isinstance(v, dict):
+                v = hashlib.md5(json.dumps(v).encode("utf-8")).digest().hex()[0:4]
             new_s = re.sub(rf"\$\{{{k}\}}", v, new_s)
             new_s = re.sub(rf"\&\{{{k}\}}", re.sub("[^0-9a-zA-Z]+", "", v), new_s)
 
@@ -298,6 +307,7 @@ class _ForEachValueFnFindInMap(_ForEachValue):
         cfn: Any,
         params: Optional[Mapping[str, Any]] = None,
         only_params: bool = False,
+        default_on_resolver_failure: bool = True,
     ) -> Any:
         if params is None:
             params = {}
@@ -361,11 +371,11 @@ class _ForEachValueFnFindInMap(_ForEachValue):
                     t_map[2].value(cfn, params, only_params)
                 )
             except _ResolveError as e:
-                if len(self._map) == 4:
+                if len(self._map) == 4 and default_on_resolver_failure:
                     return self._map[3].value(cfn, params, only_params)
                 raise _ResolveError("Can't resolve Fn::FindInMap", self._obj) from e
 
-        if len(self._map) == 4:
+        if len(self._map) == 4 and default_on_resolver_failure:
             return self._map[3].value(cfn, params, only_params)
         raise _ResolveError("Can't resolve Fn::FindInMap", self._obj)
 
@@ -452,6 +462,9 @@ class _ForEachValueRef(_ForEachValue):
                 return [x.strip() for x in allowed_values[0].split(",")]
             return allowed_values[0]
 
+        if "List" in t:
+            return [{"Fn::Select": [0, {"Ref": v}]}, {"Fn::Select": [1, {"Ref": v}]}]
+
         raise _ResolveError("Can't resolve Fn::Ref", self._obj)
 
 
@@ -489,7 +502,7 @@ class _ForEachCollection:
                 if values:
                     if isinstance(values, list):
                         for value in values:
-                            if isinstance(value, str):
+                            if isinstance(value, (str, dict)):
                                 yield value
                             else:
                                 raise _ValueError(
@@ -502,8 +515,7 @@ class _ForEachCollection:
                     )
             except _ResolveError:
                 if self._fn.hash in collection_cache:
-                    for item in collection_cache[self._fn.hash]:
-                        yield item
+                    yield from iter(collection_cache[self._fn.hash])
                 else:
                     collection_cache[self._fn.hash] = []
                     for _ in range(0, 2):
@@ -543,5 +555,4 @@ class _ForEach:
 
     def items(self, cfn: Any) -> Iterable[Tuple[str, str]]:
         items = self._collection.values(cfn, self._collection_cache)
-        for item in items:
-            yield item
+        yield from iter(items)
